require 'securerandom'
require 'active_support/all'

module LittleDutch
  class Configuration

    DEFAULTS = {
      'enabled'       => false,
      'endpoint'      => 'https://little-dutch.herokuapp.com',
      'interval'      => 60,
      'appname'       => 'app',
      'instance_name' => 'dev',
      'logfile'       => File.expand_path(File.join 'log', 'little-dutch.log')
    }

    class << self

      def option(*names)
        names.each { |n| define_option n }
      end

      def env
        if defined?(Rails)
          Rails.env
        else
          ENV['RACK_ENV'] || ENV['RAILS_ENV'] || 'development'
        end
      end

      private

      def define_option(name)
        define_method name do
          @options[name.to_s]
        end

        define_method "#{name}=" do |val|
          @options[name.to_s] = val
        end
      end

    end

    attr_writer :logger
    option :endpoint, :token, :enabled, :interval, :app_name, :instance_name
    delegate :env, to: :class

    def initialize(file)
      @options = load(file)
      FileUtils.mkdir_p File.dirname logfile if logfile.is_a? String
    end

    def instance_id
      @instance_id ||= [instance_name, uuid].join '.'
    end

    def logfile=(file)
      @logger = nil
      @options['logfile'] = file
    end

    def logfile
      output = @options['logfile']
      if output.to_s.downcase.in? %w{stdout stderr}
        output.to_s.upcase.constantize
      else
        output
      end
    end

    def logger
      @logger ||= Logger.new logfile
    end

    private

    def uuid
      @uuid ||= SecureRandom.uuid
    end

    def load(file)
      return DEFAULTS unless File.exists?(file)
      parse_yaml(File.read file).fetch(env) { {} }.reverse_merge DEFAULTS
    end

    def parse_yaml(contents)
      YAML.load ERB.new(contents).result
    end

  end
end
